import * as Minecraft from "@minecraft/server"
import * as MinecraftUi from "@minecraft/server-ui"


Minecraft.world.events.beforeItemUse.subscribe((data) => {
    let player = data.source
    if(data.item.typeId == "warps:warp_menu"){
        let actionForm = new MinecraftUi.ActionFormData()
        .title("§r§dWarp Menu§r")
        
        let warps = []
        for(let warp of Minecraft.world.scoreboard.getObjective("warps:data").getParticipants()){
            warp = JSON.parse(warp.displayName.replaceAll("\\", ""))
            if(!warp?.name || !warp?.x.toString() || !warp?.y.toString() || !warp?.z.toString() || !warp?.dimension) continue
            actionForm.button(`§r${warp.name}§r\n§7X: §8${warp.x} §7Y: §8${warp.y} §7Y: §8${warp.z}§r ${warp.dimension.replace("overworld", "§r§2Overworld§r").replace("nether", "§r§cNether§r").replace("the_end", "§r§5The End§r")}`)
            warps.push(warp)
        }

        actionForm
        .button("§r§cClose Menu§r")
        .show(player).then((res) => {
            if(res.canceled) return player.tell(`§r§8[§dWarps!§8] §cCanceled Warp Teleport!§r`)

            Minecraft.world.getDimension(warps[res.selection].dimension).runCommandAsync(`tp "${player.name}" ${warps[res.selection].x} ${warps[res.selection].y} ${warps[res.selection].z}`)
        })
    }
})

Minecraft.world.events.tick.subscribe(() => {
    for(let player of Minecraft.world.getPlayers()) if(player.hasTag("warps:settings")){
        player.removeTag("warps:settings")

        new MinecraftUi.ActionFormData()
        .title("§r§dWarps §8Settings§r")
        .button("§r§aAdd Warp§r")
        .button("§r§cRemove Warp§r")
        .show(player).then((res) => {
            if(res.canceled) return

            if(res.selection == 0){
                new MinecraftUi.ModalFormData()
                .title("§r§8Add §dWarp§r")
                .textField("Name", "Warp 0815 (Required, String)")
                .textField("X Coordinate", `${Math.round(player.location.x)} (Required, Number)`, `${Math.round(player.location.x)}`)
                .textField("Y Coordinate", `${Math.round(player.location.y)} (Required, Number)`, `${Math.round(player.location.y)}`)
                .textField("Z Coordinate", `${Math.round(player.location.z)} (Required, Number)`, `${Math.round(player.location.z)}`)
                .textField("Dimension", `${player.dimension.id.replace("minecraft:", "")} (Required, String)`, `${player.dimension.id.replace("minecraft:", "")}`)
                .show(player).then((res) => {
                    if(res.isCanceled) return player.tell(`§r§8[§dWarps!§8] §cCanceled Warp Add!§r`)
                    if(!res.formValues[0] || !res.formValues[1] || !res.formValues[2] || !res.formValues[3] || !res.formValues[4]) return player.tell(`§r§8[§dWarps!§8] §7You need to fill up everything what is required!§r`)
                    if(isNaN(res.formValues[1]) || isNaN(res.formValues[2]) || isNaN(res.formValues[3])) return player.tell(`§r§8[§dWarps!§8] §7The Coordinates must be a Number!§r`)
                    if(res.formValues[4].toLowerCase() !== "overworld" && res.formValues[4].toLowerCase() !== "nether" && res.formValues[4].toLowerCase() !== "the_end") return player.tell(`§r§8[§dWarps!§8] §7The Dimension must be one of these: "overworld", "nether", "the_end"!§r`)

                    res.formValues[0] = res.formValues[0].replace('"', "'")

                    player.runCommandAsync(`scoreboard players set "{\\"x\\": ${res.formValues[1]}, \\"y\\": ${res.formValues[2]}, \\"z\\": ${res.formValues[3]}, \\"dimension\\": \\"${res.formValues[4].toLowerCase()}\\", \\"name\\": \\"${res.formValues[0]}\\"}" warps:data 1`)
                    player.tell(`§r§8[§dWarps!§8] §7Added Warp "${res.formValues[0]}" at position X: ${res.formValues[1]}, Y: ${res.formValues[2]}, Z: ${res.formValues[3]}!§r`)
                })
            }else if(res.selection == 1){
                let actionForm = new MinecraftUi.ActionFormData()
                .title("§r§4Delete §dWarp§r")

                let warps = []
                for(let warpo of Minecraft.world.scoreboard.getObjective("warps:data").getParticipants()){
                    let warp = JSON.parse(warpo.displayName.replaceAll("\\", ""))
                    warp.data = warpo.displayName
                    if(!warp?.name || !warp?.x.toString() || !warp?.y.toString() || !warp?.z.toString() || !warp?.dimension) continue
                    actionForm.button(`§r${warp.name}§r\n§7X: §8${warp.x} §7Y: §8${warp.y} §7Y: §8${warp.z}§r ${warp.dimension.replace("overworld", "§r§2Overworld§r").replace("nether", "§r§cNether§r").replace("the_end", "§r§5The End§r")}`)
                    warps.push(warp)
                }

                actionForm.show(player).then((res) => {
                    if(res.canceled) return player.tell(`§r§8[§dWarps!§8] §cCanceled Warp Delete!§r`)

                    warps = warps[res.selection]
                    new MinecraftUi.MessageFormData()
                    .title(`§r§4Warp Delete§r`)
                    .body(`§r§cAre you sure that you want to delete this Warp for ever?§r\n§8Name: §7${warps.name}\n§8X: §7${warps.x}\n§8Y: §7${warps.y}\n§8Z: §7${warps.z}\n§8Dimension: §7${warps.dimension}`)
                    .button1("§r§cYes§r")
                    .button2("§r§aNo§r")
                    .show(player).then((res) => {
                        if(res.selection == 1){
                            player.runCommandAsync(`scoreboard players reset "${warps.data}" warps:data`)
                            return player.tell(`§r§8[§dWarps!§8] §cDeleted Warp "${warps.name}"!§r`)
                        }else{
                            return player.tell(`§r§8[§dWarps!§8] §aNot deleted Warp "${warps.name}"!§r`)
                        }
                    })
                })
            }
        })
    }
})

Minecraft.world.events.playerJoin.subscribe(() => {
    try{
        Minecraft.world.scoreboard.addObjective("warps:data", "Warps Data")
    }catch{}
})

Minecraft.system.events.beforeWatchdogTerminate.subscribe((data) => data.cancel = true)